//---------------------------------------------------------------------------
// Burner - Arcade emulator interface
// Copyright (c) 2001 Dave (www.finalburn.com), all rights reserved.

// This refers to all code except where stated otherwise
// (e.g. unzip and zlib code)

// You can use, modify and redistribute this code freely as long as you
// don't do so commercially. This copyright notice must remain with the code.
// If your program uses this code, you must either distribute or link to the
// source code. If you modify or improve this code, you must distribute
// the source code improvements.

// Dave
// Homepage: www.finalburn.com
// E-mail:  dave@finalburn.com
//---------------------------------------------------------------------------

// Main module
#include "app.h"

#ifdef _MSC_VER
 #ifdef _DEBUG
  #include <crtdbg.h>
 #endif
#endif

#include "kailleraclient.h"

//#define APP_DEBUG_LOG // log debug messages to zzd.txt

HINSTANCE hAppInst=NULL; // Application Instance
HACCEL hAccel=NULL;
HINSTANCE hImm32=NULL;
static char *szCmdLine="";
int nAppVirtualFps=600; // App fps * 10
int nUseSysMemory=-1;
int bAppBassFilter=0;
int nMonAspectX = 4;
int nMonAspectY = 3;
int nAppShowCmd=0;
int nAppThreadPriority = THREAD_PRIORITY_NORMAL;
char szAppBurnVer[16]="";
char szAppExeName[EXE_NAME_SIZE + 1];
bool bCmdOptUsed = 0;

// Show a message box with an error message
int AppError(char *szText,int bWarning)
{
  AudSoundStop();
  MessageBox(hScrnWnd,szText,
    bWarning ? APP_TITLE " " :  APP_TITLE " ",
    MB_OK | (bWarning?MB_ICONWARNING:MB_ICONERROR)
  );
  AudSoundPlay();
  return 0;
}

static int AppInit()
{
  int nRet=0;

  kailleraInit();

  // Load config for the application
  nRet=ConfigAppLoad(); if (nRet!=0) ConfigAppSave(); // create initial config file

  // Set the thread priority
  SetThreadPriority(GetCurrentThread(),nAppThreadPriority);

  // Start any wave logging here if needed
  bCheatsAllowed=1;

  ComputeGammaLUT();

  BurnLibInit(); // Init the Burn library

  MediaInit();

  //ayeye
  hAccel=LoadAccelerators(hAppInst, MAKEINTRESOURCE(IDR_ACCELERATOR));
  hImm32=LoadLibrary("imm32.dll");

  return 0;
}

static int AppExit()
{
  kailleraShutdown();
  DrvExit(); // Make sure any game driver is exitted
  MediaExit();
  BurnLibExit(); // Exit the Burn library
  WaveLogStop(); // Stop any wave logging
  ConfigAppSave(); // Save config for the application

  //ayeye
  if(hAccel!=NULL) { DestroyAcceleratorTable(hAccel); hAccel=NULL;}
  if(hImm32!=NULL) { FreeLibrary(hImm32); hImm32=NULL; }

  return 0;
}

int AppMessage(MSG *pMsg)
{

  if (kNetGame && IsDialogMessage(hPanelDlg,pMsg)) return 0;
  if (IsDialogMessage(hInpdDlg,pMsg)) return 0;
  if (IsDialogMessage(hInpsDlg,pMsg)) return 0;
  if (IsDialogMessage(hInpcDlg,pMsg)) return 0;
  if (IsDialogMessage(hInpCheatDlg,pMsg)) return 0;
  if (IsDialogMessage(hInpDIPSWDlg,pMsg)) return 0;
  return 1; // Didn't process this message
}

#ifdef APP_DEBUG_LOG
static FILE *DebugFile=NULL;
// Debug printf to a file
static int __cdecl AppDebugPrintf(char *szFormat,...)
{
  va_list Arg; va_start(Arg,szFormat);

  if (DebugFile!=NULL) { vfprintf(DebugFile,szFormat,Arg); fflush(DebugFile); }
  va_end(Arg); return 0;
}
#endif

int ProcessCmdLine()
{
	unsigned int i;
	int nOptX = 0, nOptY = 0, nOptD = 0;
	char szGameName[32] = "", szOpt[2] = "";

	sscanf(szCmdLine, "%32s %2s %i x %i x %i", szGameName, szOpt, &nOptX, &nOptY, &nOptD);

	if (strlen(szGameName)) {
		bool bFullscreen = 1;
		for (i = 0; i < nBurnDrvCount; i++) {
			nBurnDrvSelect = i;
			if (strcmp(BurnDrvText(0), szGameName) == 0) {
				bCmdOptUsed = 1;

				if (strcmp(szOpt, "-r") == 0) {
					if (nOptX && nOptY) {
						nVidWidth = nOptX;
						nVidHeight = nOptY;
					}
					if (nOptD) {
						nVidDepth = nOptD;
					}
				} else {
					if (strcmp(szOpt, "-a") == 0) {
						bVidArcaderes = 1;
					} else {
						if (strcmp(szOpt, "-w") == 0) {
							bFullscreen = 0;
						}
					}
				}

				DrvInit(i);
				if (bDrvOkay && bFullscreen) {
					nVidFullscreen = 1;
				}
				PostMessage(NULL, WM_APP + 0, 0, 0);
				break;
			}
		}
		if (i == nBurnDrvCount) {
			char szErrorString[128];
			sprintf(szErrorString, "%s ڲ Final Burn Alpha. ֧", szGameName);
			AppError(szErrorString, 0);
			return 1;
		}
	}
	return 0;
}

// Main program entry point
int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nShowCmd)
{
  int nRet=0;

  (void)hPrevInstance; // Unused parameters
  hAppInst=hInstance;
  nAppShowCmd=nShowCmd;

  AppDirectory(); // Set current directory to be the applications directory

  // Make version string
  if (nBurnVer & 0xFF) {
    // private version (alpha)
    sprintf (szAppBurnVer, "%x.%x.%x.%x İ", nBurnVer >> 20, (nBurnVer >> 16) & 0x0F, (nBurnVer >> 8) & 0xFF, nBurnVer & 0xFF);
  } else {
    // public version
	sprintf (szAppBurnVer, "%x.%x.%x İ", nBurnVer >> 20, (nBurnVer >> 16) & 0x0F, (nBurnVer >> 8) & 0xFF);
  }

  // Make sure there are roms and cfg subdirectories
  CreateDirectory("roms",NULL);
  CreateDirectory("cfg" ,NULL);
  CreateDirectory("state",NULL);
  CreateDirectory("cheat",NULL);
  CreateDirectory("dipinfo",NULL);
  CreateDirectory("previews",NULL);
  CreateDirectory("sshots",NULL);
  CreateDirectory("wav",NULL);


#ifdef APP_DEBUG_LOG
  DebugFile=fopen("zzd.txt","wt"); // Log debug to a file
  dprintf=AppDebugPrintf; // Redirect Burn library debug to our function
#endif

	InitCommonControls();

	if (lpCmdLine!=NULL) szCmdLine=lpCmdLine;

	nRet=AppInit(); // Init the application
	if (nRet == 0) {
		if (!(ProcessCmdLine())) {
			RunMessageLoop(); // Run the application message loop
		}
	}
	AppExit(); // Exit the application

#ifdef APP_DEBUG_LOG
  if (DebugFile!=NULL) fclose(DebugFile);  DebugFile=NULL;
#endif

#ifdef _MSC_VER
 #ifdef _DEBUG
  _CrtSetDbgFlag(_CRTDBG_LEAK_CHECK_DF ); // Check for memory leaks
 #endif
#endif

  return 0;
}
